from user import User, UserSessionKey, Tag, UserTag
from core import error
from db import createsession, engine

from sqlalchemy.orm.exc import MultipleResultsFound, NoResultFound

def auth_required(func):

	def wrapped(request, data, *args, **kwargs):

		auth_header = request.getHeader('Authorization')

		if auth_header is None:
			return error(request, 'Not authenticated.', http_code=401)

		header_elements = auth_header.split()
		name = header_elements[0]
		value = header_elements[1]

		if name != 'Token':
			return error(request, 'Not authorized.', http_code=403)

		with createsession(engine) as session:
			try:
				usersessionkey = session.query(UserSessionKey).filter_by(key=value).one()
			except NoResultFound:
				return error(request, 'Not authorized.', http_code=403)
			except MultipleResultsFound:
				return error(request, 'Auth error', http_code=500)

			if not usersessionkey.is_valid():
				return error(request, 'Expired.', http_code=401)
			
		user = usersessionkey.user

		return func(request, data, user, *args, **kwargs)

	return wrapped


def has_tag(tag):

	def wrapped(func):
		def verify(request, user, *args, **kwargs):
			if user.has_tag(tag):
				return func(*args, **kwargs)
			else:
				return error(request, 'Not authorized.', http_code=403)
		return verify

	return wrapped