from sqlalchemy import Column, Integer, String, DateTime, Text

import hashlib
import uuid
from datetime import datetime, timedelta

from db import models, ForeignKey, createsession, engine, attach_session
from core import defaults

class User(models.Model):

	username = Column(String(50), unique=True)
	password = Column(String(64))
	email = Column(String(255))
	created = Column(DateTime, default=datetime.utcnow)


	def change_password(self, new_password):
		self.password = hashlib.sha256(new_password).hexdigest()

	def check_password(self, password):
		if self.password == hashlib.sha256(password).hexdigest():
			return True
		return False

	def create_session_key(self):

		key = str(uuid.uuid4())

		with createsession(engine) as session:
			sessionkey = UserSessionKey(user=self, key=key)
			session.add(sessionkey)
			session.commit()

		return key

	@attach_session
	def has_tag(self, tag, session):
		if isinstance(tag, str):
			try:
				tag = session.query(Tag).filter_by(name=tag).one()
			except:
				return False

		try:
			usertag = session.query(UserTag).filter_by(user=self, tag=tag).one()
			return True
		except:
			return False

	@attach_session
	def set_tag(self, tag, session):
		if isinstance(tag, str):
			try:
				tag = session.query(Tag).filter_by(name=tag).one()
			except:
				raise Exception('Tag error.')

		usertag = UserTag(user=self, tag=tag)
		session.add(usertag)

	@attach_session
	def delete_tag(self, tag, session):
		if isinstance(tag, str):
			try:
				tag = session.query(Tag).filter_by(name=tag).one()
			except:
				raise Exception('Tag error.')

		try:
			usertag = session.query(UserTag).filter_by(user=self, tag=tag).one()
			session.delete(usertag)
		except:
			raise Exception('Error.')


class Tag(models.Model):
	name = Column(String(50))
	info = Column(Text)

class UserTag(models.Model):
	user = ForeignKey(User)
	tag = ForeignKey(Tag)

class UserSessionKey(models.Model):
	user = ForeignKey(User)
	key = Column(String(255))
	created = Column(DateTime, default=datetime.utcnow)
	last_used = Column(DateTime, default=datetime.utcnow)

	def touch(self):
		with createsession(engine) as session:
			self.last_used = datetime.utcnow()
			session.add(self)
			session.commit()

	def is_valid(self):
		max_vality = self.last_used + timedelta(minutes=defaults.TIMEOUT_MINUTES)

		if max_vality < datetime.utcnow():
			return False

		return True