import json
import importlib

import defaults

from utils import Operation
from conf import settings

from core import error

class Dispatcher:
	
	def __init__(self, oplist=[]):
		ops = self._opdict = {}

		for operation in oplist:
			ops[operation.name] = operation.function


	def loadallmodules(self):

		enabled_modules = settings.get('modules') or []

		for module in enabled_modules:

			try:
				mod = importlib.import_module(defaults.MODULES_NAME + module + defaults._SETTINGS)
				uri = getattr(mod, 'uri', '')
				operations = getattr(mod, 'operations', [])				
				self.adduri(uri, operations)
			except ImportError:
				raise



	def adduri(self, uri, oplist):

		ops = self._opdict

		if uri == '':
			for operation in oplist:
				self._opdict[operation.name] = operation.function

			return


		if uri in ops and isinstance(ops[uri], Dispatcher):
			for operation in oplist:
				ops[uri].addop(operation)

		else:
			ops[uri] = Dispatcher(oplist)



	def addop(self, operation):
		self._opdict[operation.name] = operation.function



	def process(self, request, urilist):
		ops = self._opdict

		# check URI
		if len(urilist) > 0 and urilist[0] in ops:
			
			if isinstance(ops[urilist[0]], Dispatcher):
				return ops[urilist[0]].process(request, urilist[1:])

			data = json.loads(request.content.getvalue())
			return ops[urilist[0]] (request, data)

		# check op
		try:
			data = json.loads(request.content.getvalue())
		except ValueError:
			data = {}

		if 'op' in data and data['op'] in ops:
			return ops[data['op']] (request, data)
		
		if len(urilist) == 0 and '' in ops:
			return ops[''] (request, data)

		return error(request, 'Request not found.', http_code=404)
