from sqlalchemy.orm import sessionmaker, relationship, backref
from sqlalchemy import create_engine
from sqlalchemy import Column, Integer
from sqlalchemy import ForeignKey as ForeignKey_SA

from core import defaults
from conf import settings


engine = None
session = None


def createengine():
	dbconf = settings.get('database')

	if dbconf is None:
		print 'Database information not found, using in-memory database.'

		return create_engine('sqlite:///:memory:')

	else:
		encoding = dbconf['encoding'] if 'encoding' in dbconf else defaults.DB_ENCODING
		enginetype = dbconf['enginetype']
		destination = dbconf['destination']

		if enginetype == 'sqlite':
			dbstring = '%s:///%s' % (enginetype, destination)
		else:
			username = dbconf['username']
			password = dbconf['password']
			db = dbconf['db']
			port = dbconf['port']

			dbstring = '%s://%s:%s@%s:%s/%s' % (enginetype, username, password, destination, port, db)

		return create_engine(dbstring, encoding=encoding)


def createsession(eng=engine):
	Session = sessionmaker(bind=eng)
	return Session()


def createtables():
	pass


def attach_session(func):

	def wrapped(session=None, *args, **kwargs):

		if session is None:
			with createsession(engine) as session:
				return func(session=session, *args, **kwargs)
				session.commit()
		else:
			return func(session=session, *args, **kwargs)

	return wrapped

def ForeignKey(origin, model):
	cls = origin.__class__
	attrname = model.__name__.lower() + '_id'
	column = Column(Integer, ForeignKey_SA(model.__tablename__ + '.id'))

	if not hasattr(cls, attrname)
		setattr(cls, attrname, column)

	else:
		i = 1

		while not hasattr(cls, attrname + str(i)):
			i = i + 1

		attrname = attrname + str(i)

		setattr(cls, attrname, column)


	fk_name = cls.__name__ + '.' + attrname

	return relationship(model.__name__,foreign_keys=fk_name)
