import json

from db import models, ForeignKey, createsession, engine, attach_session
from auth import User
from core import defaults, error
from accounts import Currency, Account, Movement

from datetime import datetime

from sqlalchemy.orm.exc import MultipleResultsFound, NoResultFound
from sqlalchemy import Column, String, Numeric, DateTime, Text, Boolean, Integer

import urllib2
import blockchain.formatstrings as blc

class CurrencyWallet(models.Model):
	currency = ForeignKey(Currency)
	identifier = Column(String(100))
	password = Column(String(100))
	defaultrate = Column(Numeric(20, 10))

	@attach_session
	def get_rate(self, session):
		stock = self.currency.stock

		url = blc['getbalance'].format(identifier=identifier, password=password)

		response = urllib2.urlopen(url)
		data = json.loads(response.read())

		if 'error' in data:
			raise Exception(data['error'])

		if 'balance' in data:
			balance = Decimal(data['balance'])
		else:
			raise Exception('No balance.')

		rate = (balance / Decimal('1e8')) / stock

		if rate == Decimal(0):
			rate = self.defaultrate

		return rate

	@staticmethod
	@attach_session
	def getinfo(currency, session):
		if isinstance(currency, str):
			try:
				currency = session.query(Currency).filter_by(name=currency).one()
			except NoResultFound:
				raise Exception('Invalid currency.')

		try:
			walletinfo = session.query(CurrencyWallet).filter_by(currency=currency).one()
		except NoResultFound:
			raise Exception('No wallet info found.')
		except MultipleResultsFound:
			raise Exception('Wallet info error.')

		response = {
			"currency": currency.name,
			"identifier": walletinfo.identifier,
			"password": walletinfo.password,
			"defaultrate": walletinfo.defaultrate
		}

		return response


class AccountWallet(models.Model):
	account = Column(String(255))
	address = Column(String(50))

class IncomingTransactions(models.Model):
	txhash = Column(String(255))
	account = Column(String(255))

class MovementQueue(models.Model):
	timestamp = Column(DateTime)
	movement = ForeignKey(Movement)
	orig = Column(String(255))
	dest = Column(String(255))
	user = ForeignKey(User)
	status = Column(String(50))

	@staticmethod
	@attach_session
	def next(status, session):
		try:
			item = session.query(MovementQueue).filter_by(status=status).order_by('-timestamp').first()
		except NoResultFound:
			return None

		return item

	@staticmethod
	@attach_session
	def put(status, movement, orig, dest, user, session):

		if isinstance(orig, Account):
			orig = orig.name

		if isinstance(dest, Account):
			dest = dest.name

		item = MovementQueue(timestamp=movement.timestamp, movement=movement, orig=orig, dest=dest, user=user, status=status)
		session.add(item)
		pass